@@ -1,14 +1,13 @@
 #include "sampling.h"
 #include "family.h"
+#include "betapriorfamily.h"
 #include "bas-glm.h"
 
 
-
 SEXP glm_mcmcbas(SEXP Y, SEXP X, SEXP Roffset, SEXP Rweights, 
 		 SEXP Rprobinit, SEXP Rmodeldim, 
-		 SEXP modelprior, SEXP Rbestmodel,  SEXP Rbestmarg,SEXP plocal, 
+		 SEXP modelprior, SEXP betaprior, SEXP Rbestmodel,  SEXP Rbestmarg,SEXP plocal, 
 		 SEXP BURNIN_Iterations,
-		 SEXP Ra, SEXP Rb, SEXP Rs,
 		 SEXP family, SEXP Rcontrol,
 		 SEXP Rupdate, SEXP Rlaplace) 
 {
@@ -38,10 +37,14 @@ SEXP glm_mcmcbas(SEXP Y, SEXP X, SEXP Roffset, SEXP Rweights,
 	double *probs, MH=0.0, prior_m=1.0,shrinkage_m, logmargy, postold, postnew;
 	int i, m, n, pmodel_old, *bestmodel;
 	int mcurrent, n_sure;
+	
 	glmstptr *glmfamily;
-
 	glmfamily = make_glmfamily_structure(family);
 
+	betapriorptr *betapriorfamily;
+	betapriorfamily = make_betaprior_structure(betaprior, family);
+
+
 	//get dimsensions of all variables 
 	int p = INTEGER(getAttrib(X,R_DimSymbol))[1];
 	int k = LENGTH(modelprobs);
@@ -79,7 +82,8 @@ SEXP glm_mcmcbas(SEXP Y, SEXP X, SEXP Roffset, SEXP Rweights,
 	GetModel_m(Rmodel_m, model, p);
 	//evaluate logmargy and shrinkage
 	SEXP glm_fit = PROTECT(glm_FitModel(X, Y, Rmodel_m, Roffset, Rweights,
-					    glmfamily, Rcontrol, Ra, Rb, Rs, Rlaplace));	
+					    glmfamily, Rcontrol, Rlaplace,
+					    betapriorfamily));	
 	prior_m  = compute_prior_probs(model,pmodel,p, modelprior);
 
 	logmargy = REAL(getListElement(getListElement(glm_fit, "lpy"),"lpY"))[0];
@@ -133,7 +137,8 @@ SEXP glm_mcmcbas(SEXP Y, SEXP X, SEXP Roffset, SEXP Rweights,
 		  GetModel_m(Rmodel_m, model, p);
 
 		  glm_fit = PROTECT(glm_FitModel(X, Y, Rmodel_m, Roffset, Rweights,
-						 glmfamily, Rcontrol, Ra, Rb, Rs, Rlaplace));	
+						 glmfamily, Rcontrol, Rlaplace,
+						 betapriorfamily));	
 		  prior_m = compute_prior_probs(model,pmodel,p, modelprior);
 
 		  logmargy = REAL(getListElement(getListElement(glm_fit, "lpy"),"lpY"))[0];
@@ -182,7 +187,7 @@ SEXP glm_mcmcbas(SEXP Y, SEXP X, SEXP Roffset, SEXP Rweights,
 
 	// Compute marginal probabilities  
 	mcurrent = nUnique;
-	Rprintf("NumUnique Models Accepted %d \n", nUnique);
+	//	Rprintf("NumUnique Models Accepted %d \n", nUnique);
 	compute_modelprobs(modelprobs, logmarg, priorprobs,mcurrent);
 	compute_margprobs(modelspace, modeldim, modelprobs, probs, mcurrent, p);        
 
@@ -212,7 +217,8 @@ SEXP glm_mcmcbas(SEXP Y, SEXP X, SEXP Roffset, SEXP Rweights,
 	GetModel_m(Rmodel_m, model, p);
 
 	glm_fit = PROTECT(glm_FitModel(X, Y, Rmodel_m, Roffset, Rweights,
-				       glmfamily, Rcontrol, Ra, Rb, Rs, Rlaplace));	
+				       glmfamily, Rcontrol, Rlaplace,
+				       betapriorfamily));	
 	prior_m = compute_prior_probs(model,pmodel,p, modelprior);
 	logmargy = REAL(getListElement(getListElement(glm_fit, "lpy"),"lpY"))[0];
 	shrinkage_m = REAL(getListElement(getListElement(glm_fit, "lpy"),	
@@ -240,7 +246,7 @@ SEXP glm_mcmcbas(SEXP Y, SEXP X, SEXP Roffset, SEXP Rweights,
 	    compute_modelprobs(modelprobs, logmarg, priorprobs,mcurrent);
 	    compute_margprobs(modelspace, modeldim, modelprobs, probs, mcurrent, p);        
 	    if (update_probs(probs, vars, mcurrent, k, p) == 1) {
-	      Rprintf("Updating Model Tree %d \n", m);
+	      //	      Rprintf("Updating Model Tree %d \n", m);
 	      update_tree(modelspace, tree, modeldim, vars, k,p,n,mcurrent, modelwork);     
 	    }
 	  }
@@ -274,10 +280,10 @@ SEXP glm_mcmcbas(SEXP Y, SEXP X, SEXP Roffset, SEXP Rweights,
 	SET_STRING_ELT(ANS_names, 6, mkChar("deviance"));
 
 	SET_VECTOR_ELT(ANS, 7, beta);
-	SET_STRING_ELT(ANS_names, 7, mkChar("coefficients"));
+	SET_STRING_ELT(ANS_names, 7, mkChar("mle"));
 
 	SET_VECTOR_ELT(ANS, 8, se);
-	SET_STRING_ELT(ANS_names, 8, mkChar("se"));
+	SET_STRING_ELT(ANS_names, 8, mkChar("mle.se"));
 
 	SET_VECTOR_ELT(ANS, 9, shrinkage);
 	SET_STRING_ELT(ANS_names, 9, mkChar("shrinkage"));
@@ -307,7 +313,7 @@ SEXP glm_mcmcbas(SEXP Y, SEXP X, SEXP Roffset, SEXP Rweights,
 	setAttrib(ANS, R_NamesSymbol, ANS_names);
 	
 	PutRNGstate();
-    UNPROTECT(nProtected);
+	UNPROTECT(nProtected);
 	return(ANS);  
 }
 
