@@ -26,9 +26,8 @@ void  update_Cov(double *Cov, double *priorCov, double *SSgam, double *marg_prob
 
 void insert_model_tree(struct Node *tree, struct Var *vars,  int n, int *model, int num_models);
  
-SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rprobinit, SEXP Rmodeldim, SEXP incint, SEXP Ralpha,SEXP method, SEXP modelprior, SEXP Rupdate, SEXP Rbestmodel, SEXP Rbestmarg, SEXP plocal, SEXP BURNIN_Iterations, SEXP MCMC_Iterations, SEXP LAMBDA, SEXP DELTA)
+SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim, SEXP incint, SEXP Ralpha,SEXP method, SEXP modelprior, SEXP Rupdate, SEXP Rbestmodel, SEXP Rbestmarg, SEXP plocal, SEXP BURNIN_Iterations, SEXP MCMC_Iterations, SEXP LAMBDA, SEXP DELTA)
 {
-  SEXP   Rse_m, Rcoef_m, Rmodel_m; 
 
   SEXP   RXwork = PROTECT(duplicate(X)), RYwork = PROTECT(duplicate(Y));
 
@@ -53,19 +52,20 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rprobinit, SEXP Rmodeldim, SEXP incint, SEXP R
   SEXP logmarg = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
   SEXP sampleprobs = PROTECT(allocVector(REALSXP, nModels)); ++nProtected;
   SEXP NumUnique = PROTECT(allocVector(INTSXP, 1)); ++nProtected;
-
-  double *Xwork, *Ywork, *coefficients,*probs, shrinkage_m, *MCMC_probs,
-    SSY, yty, ybar, mse_m, *se_m, MH=0.0, prior_m=1.0, *real_model, 
+  SEXP Rse_m = NULL, Rcoef_m = NULL, Rmodel_m;
+  
+  double *Xwork, *Ywork, *wts, *coefficients,*probs, shrinkage_m, *MCMC_probs,
+    SSY, yty, mse_m, *se_m, MH=0.0, prior_m=1.0, *real_model, 
     R2_m, RSquareFull, alpha, prone, denom, logmargy, postold, postnew;
   int nobs, p, k, i, j, m, n, l, pmodel, pmodel_old, *xdims, *model_m, *bestmodel, *varin, *varout;
   int mcurrent,  update, n_sure;
   double  mod, rem, problocal, *pigamma,  eps, *hyper_parameters;
   double *XtX, *XtY, *XtXwork, *XtYwork, *SSgam, *Cov, *priorCov, *marg_probs;
-  double one=1.0, zero=0.0, lambda,  delta; 
+  double  lambda,  delta, one=1.0; 
  
-  int inc=1, p2;
+  int inc=1;
   int *model, *modelold, bit, *modelwork, old_loc, new_loc;	
-  char uplo[] = "U", trans[]="T";
+  //  char uplo[] = "U", trans[]="T";
   struct Var *vars;	/* Info about the model variables. */
   NODEPTR tree, branch;
 
@@ -90,44 +90,12 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rprobinit, SEXP Rmodeldim, SEXP incint, SEXP R
 
   Ywork = REAL(RYwork);
   Xwork = REAL(RXwork);
+  wts = REAL(Rweights);
 
  
  /* Allocate other variables.  */ 
-  XtX  = (double *) R_alloc(p * p, sizeof(double));
-  XtXwork  = (double *) R_alloc(p * p, sizeof(double));
-  XtY = vecalloc(p);  
-  XtYwork = vecalloc(p);
 
-
-  /* create X matrix */
-  for (j=0, l=0; j < p; j++) {
-    for (i = 0; i < p; i++) {
-      XtX[j*p + i] = 0.0;
-    }
-    /*    for (i=0; i < nobs; i++) {
-       Xmat[i][j] =  REAL(X)[l];
-       Xwork[l] = Xmat[i][j];
-       l = l + 1; 
-       } */
-  }
-  //  PROTECT(Rprobs = NEW_NUMERIC(p));
-  //initprobs = REAL(Rprobinit);
-
-
- p2 = p*p;
- ybar = 0.0; SSY = 0.0; yty = 0.0; 
-
- 
- F77_NAME(dsyrk)(uplo, trans, &p, &nobs, &one, &Xwork[0], &nobs, &zero, &XtX[0], &p); 
- yty = F77_NAME(ddot)(&nobs, &Ywork[0], &inc, &Ywork[0], &inc);
- for (i = 0; i< nobs; i++) {
-     ybar += Ywork[i];
-  }
-
-  ybar = ybar/ (double) nobs;
-  SSY = yty - (double) nobs* ybar *ybar;
-
-  F77_NAME(dgemv)(trans, &nobs, &p, &one, &Xwork[0], &nobs, &Ywork[0], &inc, &zero, &XtY[0],&inc);
+  PrecomputeData(Xwork, Ywork, wts, &XtXwork, &XtYwork, &XtX, &XtY, &yty, &SSY, p, nobs);
   
   alpha = REAL(Ralpha)[0];
 
@@ -179,23 +147,8 @@ SEXP mcmcbas(SEXP Y, SEXP X, SEXP Rprobinit, SEXP Rmodeldim, SEXP incint, SEXP R
 
   /*  Rprintf("Fit Full Model\n"); */
 
-  if (nobs <= p) {RSquareFull = 1.0;}
-  else {
-  PROTECT(Rcoef_m = NEW_NUMERIC(p));
-  PROTECT(Rse_m = NEW_NUMERIC(p));
-  coefficients = REAL(Rcoef_m);  
-  se_m = REAL(Rse_m);
-  memcpy(coefficients, XtY,  p*sizeof(double));
-  memcpy(XtXwork, XtX, p2*sizeof(double));
-  memcpy(XtYwork, XtY,  p*sizeof(double));
 
-  mse_m = yty; 
-  cholreg(XtYwork, XtXwork, coefficients, se_m, &mse_m, p, nobs);  
-
-  /*olsreg(Ywork, Xwork,  coefficients, se_m, &mse_m, &p, &nobs, pivot,qraux,work,residuals,effects,v, betaols); */
-  RSquareFull =  1.0 - (mse_m * (double) ( nobs - p))/SSY;
-  UNPROTECT(2);
-  }
+  RSquareFull = CalculateRSquareFull(XtY, XtX, XtXwork, XtYwork, Rcoef_m, Rse_m, p, nobs, yty, SSY);
 
 
   /* fill in the sure things */
